package com.sdi.wap.demo.site;
import com.sdi.wap.*;
import com.sdi.wap.demo.bean.*;
import com.sdi.wap.demo.field.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
/**
 * Define static names for each page in the system.
 */
public class DemoSite
	extends SiteImpl
	implements Site
{
	public final static Site instance = new DemoSite();
	/**
	 * The name of a reserved session attribute the the site's Login 
	 * State sets with an instance of the currently logged in Account.
	 */
	public final static String accountParameterName = "account";
	/**
	 * This site's document root directory
	 */
	public final static String documentRoot = "/sdi/wap";
	/**
	 * This site's context/zone name
	 */
	public final static String contextName = "/wap";
	/**
	 * The name of this servlet
	 */
	public final static String servletName = "/demo";

	public final static NumberFormat moneyFmt = NumberFormat.getCurrencyInstance();
	public final static SimpleDateFormat dateFmt = new SimpleDateFormat("MMM dd yyyy");
	public final static SimpleDateFormat timeFmt = new SimpleDateFormat("MMM dd yyyy hh:mm:ss a");
	public final static SimpleDateFormat monthFmt = new SimpleDateFormat("MMM");

	/**
	 * Roles
	 */
	protected final static Role RegisteredRole = 
		new RoleImpl( "Registered", Role.Everybody, "Registered Account");

	/**
	 * Static (web server based) pages
	 */
	protected static final Page Home = new StaticPage(
		"Home",
		null, Role.Everybody,
		"/index.html", "Demonstration home page"
	);
	/**
	 * Dynamic (servlet based) pages
	 */
	protected final static DynamicPage Account = new DynamicPage(
		new Op("Account"), new Op(null), ViewAccount.class,
		Home, RegisteredRole,
		"Account", "To your account"
	);
	protected final static DynamicPage Register = new DynamicPage(
		new Op("Register"), new Op(null), ViewRegistration.class,
		Home, Role.Everybody,
		"Registration", "New Account Registration"
	);
	protected final static DynamicPage EditAddress= new DynamicPage(
		new Op("EditAddress"), new Op(null), ViewAddress.class,
		Home, RegisteredRole,
		"Address", "To modify the address and/or phone number"
	);
	protected final static DynamicPage Login = new DynamicPage(
		new Op("Login"), new Op(null), ViewLogin.class,
		Home, Role.Everybody,
		"Login", "Login"
	);
	protected final static DynamicPage Logout = new DynamicPage(
		new Op("Logout"), new Op(null), ViewLogout.class,
		Home, Role.Everybody,
		"Logout", "Logout"
	);
	protected final static DynamicPage Refuse = new DynamicPage(
		new Op("Refuse"), new Op(null), ViewRefuse.class,
		Home, Role.Everybody,
		"Refuse", "Permission denied"
	);
	protected final static DynamicPage defaultPage = Login;
	protected final static DynamicPage loginPage = Login;
	protected final static DynamicPage refusePage = Refuse;

public DemoSite() 
{
	super();
}
/**
 * Get the base directory of the html server.
 * @return java.lang.String
 */
public final String getBaseDirectory() {
	return documentRoot;
}
/**
 * Get the context name
 * @return java.lang.String
 */
public final String getContextName() {
	return contextName;
}
/**
 * Get the default Page, the page that should be displayed if the servlet
 * receives an invalid request with no do= parameter.
 * @return java.lang.String
 */
public final DynamicPage getDefaultPage() {
	return Login;
}
/**
 * Get the login page; the page that guides the user through the login process.
 * @return Page
 */
public DynamicPage getLoginPage() { return Login; }
/**
 * Get the refuse Page, the page that should be displayed if the user
 * requests a resource for which permission should be denied.
 * @return Page
 */
public DynamicPage getRefusePage() { return Refuse; }
/**
 * Get the servlet name
 * @return java.lang.String
 */
public final String getServletName() {
	return servletName;
}
public String htmlLeft(View view)
{
return {{
<br><center><B>Your<br>text<br>goes<br>here</B></center>
}};
}
public String htmlPage(View view, String contents)
	throws Exception
{
	return htmlPageOpen(view) + contents + htmlPageClose(view);
}
public String htmlPageClose(View view)
{
	Object path = view.getClass().getName();
	java.util.Date date = new java.util.Date();
	return {{
</body>
</html>
}};
}
public String htmlPageOpen(View view)
	throws Exception
{
	return {{
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"
			"http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML>
<HEAD><TITLE>Superdistribution, Inc.</TITLE>
<style type="text/css"><!--
	A{text-decoration: none;}
	A:link{color: #660000; text-decoration: none }
	A:visited{color:#660000; text-decoration: none}
	A:hover{color:#ffffff; text-decoration: none; background: #660000;}
	A:active{color:#ffffff; text-decoration: none; background: #660000;}
--></style>
</head>
<BODY 
	BGCOLOR="#ffffcc" 
	TEXT="#000000" 
	LINK="#660000" 
	ALINK="#660000" 
	VLINK="#660000"
>
}};
}
public final String htmlTop(View view)
	throws Exception
{
	return {{
<h2 align=center>Demonstration Web Application</h2>
<p><table border="0" cellpadding="0" cellspacing=0 width="100%" align=center>
<TR>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">
		<A HREF="/sdi/wap/index.html" target="_top">Home</A>
	</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th><FONT SIZE="2" FACE="Arial,Helvetica,sans-serif">Etc</FONT></TH>
	<th bgcolor="#cccc66"><FONT SIZE="3" FACE="Arial,Helvetica,sans-serif">
		<A HREF="/wap/demo?do=Account" target="_top" TITLE="Your account">My&nbsp;Account</A>
	</FONT></TH>
</TR>
</TABLE>
}};
}
/**
 * Set the currently logged in account by storing it in a session attribute
 */
public void setAccount(Account account, View view)
{
	HttpSession session = view.getSession();
	session.setAttribute(accountParameterName, account);
}
}
