///////////////////////////////////////////////////////////////////////////////
// Name:        generic/vcarddlg.cpp
// Purpose:     the dialog for editing vCards
// Author:      Vadim Zeitlin
// Modified by:
// Created:     16.05.00
// RCS-ID:      $Id: vcarddlg.cpp,v 1.14 2000/08/14 15:28:43 zeitlin Exp $
// Copyright:   (c) 2000 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
// Licence:     wxWindows license
///////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#ifdef __GNUG__
    #pragma interface "vcarddlg.h"
    #pragma implementation "vcarddlg.h"
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/dynarray.h"
    #include "wx/log.h"
    #include "wx/intl.h"

    #include "wx/menu.h"

    #include "wx/dcmemory.h"
    #include "wx/bmpbuttn.h"
    #include "wx/button.h"
    #include "wx/stattext.h"
    #include "wx/statbox.h"
    #include "wx/checkbox.h"
    #include "wx/listbox.h"
    #include "wx/radiobox.h"
    #include "wx/textctrl.h"
    #include "wx/sizer.h"

    #include "wx/filedlg.h"
    #include "wx/textdlg.h"
#endif // WX_PRECOMP

#include "wx/imaglist.h"
#include "wx/datetime.h"
#include "wx/notebook.h"
#include "wx/calctrl.h"
#include "wx/statline.h"

#include "wx/minifram.h"

#ifdef WXVCARD_STANDALONE
    #include "vcard.h"
#else
    #include "wx/vcard.h"
#endif

// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------

// notebook icons
#include "ident.xpm"
#include "network.xpm"
#include "phone.xpm"

// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

// control ids
enum
{
    // the buttons ids must be consecutive in this order (Add/Edit/Delete) as
    // this is assumed in the code below
    Email_Add = 1000,
    Email_Modify,
    Email_Delete,

    Addr_Add,
    Addr_Modify,
    Addr_Delete,

    Phone_Add,
    Phone_Modify,
    Phone_Delete,

    // menu commands
    ImageMenu_Change,
    ImageMenu_Clear,
    ImageMenu_Show,

    ImageMenu_Max
};

// notebook pages
enum
{
    Page_Ident,
    Page_Network,
    Page_Address,
    Page_Phone,

    Page_Max
};

// ----------------------------------------------------------------------------
// some macros to make writing sizer-based dialog slightly less painful
// ----------------------------------------------------------------------------

#define ADD_TEXT_LABEL_TO_SIZER(sizer, label, var)  \
    var = new wxTextCtrl(panel, -1);                \
    sizer->Add(CreateLabelSizer(panel, label, var), \
               0, wxGROW | wxALL, 5)

#define ADD_TEXT_LABEL_TO_GRIDSIZER(sizer, label, var)     \
    var = new wxTextCtrl(panel, -1);                       \
    sizer->Add(new wxStaticText(panel, -1, label), 0,      \
               wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);   \
    sizer->Add(var, 1, wxGROW | wxALIGN_CENTER_VERTICAL)

#define ADD_CHECKBOX_TO_SIZER(sizer, label, var)    \
    var = new wxCheckBox(panel, -1, label);         \
    sizer->Add(var, 0, wxALIGN_CENTER_VERTICAL)

#define ADD_IMAGE_TO_SIZER(sizer, label, var)                   \
    var = new wxImageButton(panel, -1);                         \
    sizer->Add(CreateLabelSizer(panel, label, var, TRUE),       \
               1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL, 5)

// ----------------------------------------------------------------------------
// helper classes for address and phone editing
// ----------------------------------------------------------------------------

// this struct contains all the address data and can be constructed directly
// from vCard
struct wxVCardAddressData
{
    wxVCardAddressData(wxVCardAddress* addr)
    {
        if ( addr )
        {
            postoffice = addr->GetPostOffice();
            extaddr = addr->GetExtAddress();
            street = addr->GetStreet();
            city = addr->GetLocality();
            region = addr->GetRegion();
            postalcode = addr->GetPostalCode();
            country = addr->GetCountry();
            flags = addr->GetFlags();
        }
        else
        {
            flags = wxVCardAddress::Default;
        }
    }

    wxString postoffice,
             extaddr,
             street,
             city,
             region,
             postalcode,
             country;
    int flags;
};

// just like wxVCardAddressData but this one is for the phone numbers
struct wxVCardPhoneData
{
    wxVCardPhoneData(wxVCardPhoneNumber *phone)
    {
        if ( phone )
        {
            number = phone->GetNumber();

            flags = phone->GetFlags();
        }
        else
        {
            flags = 0;
        }
    }

    wxString number;
    int flags;
};

// ----------------------------------------------------------------------------
// define "template" array classes
// ----------------------------------------------------------------------------

WX_DECLARE_OBJARRAY(wxVCardAddressData, wxVCardAddresses);
WX_DECLARE_OBJARRAY(wxVCardPhoneData, wxVCardPhones);

#include "wx/arrimpl.cpp"

WX_DEFINE_OBJARRAY(wxVCardAddresses);
WX_DEFINE_OBJARRAY(wxVCardPhones);

// ----------------------------------------------------------------------------
// a dialog allowing the user to choose a date
// ----------------------------------------------------------------------------

class wxDateDialog : public wxDialog
{
public:
    wxDateDialog(wxWindow *parent,
                 const wxString& title,
                 const wxDateTime& dt);

    wxDateTime GetDate() const { return m_calendar->GetDate(); }

protected:
    // event handler for double clicking the calendar
    void OnCalendar(wxCalendarEvent& event);

    wxCalendarCtrl *m_calendar;

private:
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// a frame showing an image (and nothing else)
// ----------------------------------------------------------------------------

class ImageViewFrame : public wxMiniFrame
{
public:
    ImageViewFrame(wxWindow *parent, const wxBitmap& bitmap)
        : wxMiniFrame(parent, -1, wxEmptyString,
                      wxDefaultPosition, wxDefaultSize,
                      wxCAPTION | wxSYSTEM_MENU),
          m_bitmap(bitmap)
    {
        m_width = bitmap.GetWidth();
        m_height = bitmap.GetHeight();

        SetTitle(wxString::Format(_("Image (%dx%d)"), m_width, m_height));
        SetClientSize(m_width, m_height);
        CentreOnParent();
    }

    void OnClick(wxMouseEvent& WXUNUSED(event))
    {
        Close();
    }

    void OnPaint(wxPaintEvent& WXUNUSED(event))
    {
        int w, h;
        GetClientSize(&w, &h);

        wxPaintDC dc(this);
        dc.DrawBitmap(m_bitmap, (w - m_width)/2, (h - m_height)/2);
    }

private:
    int m_width,
        m_height;
    wxBitmap m_bitmap;

    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// a button which pops up a dialog allowing to browse for date
// ----------------------------------------------------------------------------

class wxDateBrowseButton : public wxButton
{
public:
    wxDateBrowseButton(wxTextCtrl *text, const wxString& tooltip)
        : wxButton(text->GetParent(), -1, ">>")
        {
            m_text = text;

            SetToolTip(tooltip);

            wxCoord w = 3*wxClientDC(this).GetCharWidth();
            SetSize(w, -1);
        }

    // this function is called in response to the click on the button, but may
    // be also called directly
    virtual void DoBrowse();

protected:
    // callback
    void OnButtonClick(wxCommandEvent&) { DoBrowse(); }

    wxTextCtrl *m_text;

private:
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// vCard image editing control
// ----------------------------------------------------------------------------

class wxImageButton : public wxBitmapButton
{
public:
    // ctor and dtor
    wxImageButton(wxWindow *parent, int id);
    virtual ~wxImageButton();

    // accessors
    void SetImage(const wxImage& image);
    void ClearImage();
    wxImage GetImage() const { return m_image; }

    // has the image been changed?
    bool IsDirty() const { return m_dirty; }

    // event handlers
    void OnChangeImage(wxCommandEvent& event);
    void OnClearImage(wxCommandEvent& event);
    void OnShowImage(wxCommandEvent& event);

    void OnButtonClick(wxCommandEvent& event);
    void OnRightClick(wxMouseEvent& event);

    void OnSize(wxSizeEvent& event);

private:
    // return the bitmap to show for no image
    wxBitmap GetEmptyBitmap(const wxSize& size) const;

    // set the bitmap to display
    void DoSetBitmap(const wxBitmap& bmp);

    // rescale the image if it is bigger than the button size and display ut
    void ScaleAndSetImage();

    // let the user choose the image file
    void SelectImage();

    // the default initial size of the button
    static const wxSize ms_size;

    // the image we display
    wxImage m_image;

    // the popup menu used by OnRightClick()
    wxMenu *m_menuPopup;

    // has the image been changed?
    bool m_dirty;

    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// vCard editing dialog
// ----------------------------------------------------------------------------

class wxVCardDialog : public wxDialog
{
public:
    wxVCardDialog(wxWindow *parent, wxVCard *vcard);
    virtual ~wxVCardDialog() { delete m_imagelist; }

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

protected:
    // event handlers
    void OnEmailAdd(wxCommandEvent&);
    void OnEmailModify(wxCommandEvent&);
    void OnEmailDelete(wxCommandEvent&);

    void OnAddrAdd(wxCommandEvent&);
    void OnAddrModify(wxCommandEvent&);
    void OnAddrDelete(wxCommandEvent&);

    void OnPhoneAdd(wxCommandEvent&);
    void OnPhoneModify(wxCommandEvent&);
    void OnPhoneDelete(wxCommandEvent&);

    void OnUpdateEMail(wxUpdateUIEvent&);
    void OnUpdateAddress(wxUpdateUIEvent&);
    void OnUpdatePhone(wxUpdateUIEvent&);

    // creates a vertical sizer containing the buttons used with listboxes:
    // add/modify/remove
    wxSizer *CreateLboxButtonsSizer(wxWindow *parent, int idFirstButton);

    // creates a horizontal box sizer containg a label and the control
    wxSizer *CreateLabelSizer(wxWindow *parent,
                              const wxString& label,
                              wxControl *control,
                              bool resizeable = FALSE);

    // returns the label in the lbox for this address/phone
    wxString GetAddressLabel(const wxVCardAddressData& data) const;
    wxString GetPhoneLabel(const wxVCardPhoneData& data) const;

    // adds an address/phone to the list
    void AddAddress(const wxVCardAddressData& data);
    void AddPhone(const wxVCardPhoneData& data);

private:
    // the GUI controls holding vCard data
    wxTextCtrl *m_fullname,
               *m_firstname,
               *m_lastname,
               *m_middlename,
               *m_prefixname,
               *m_suffixname,
               *m_orgname,
               *m_department,
               *m_title,
               *m_birthday,
               *m_comments,
               *m_url,
               *m_key;

    wxRadioBox *m_keyKind;

    wxListBox *m_emails,
              *m_addresses,
              *m_phones;

    wxImageButton *m_photo,
                  *m_logo;

    wxImageList *m_imagelist;

    // the card we're editing
    wxVCard *m_vcard;

    // all addresses/phones
    wxVCardAddresses m_addrData;
    wxVCardPhones m_phoneData;

    bool m_emailDirty,
         m_addrDirty,
         m_phoneDirty;

    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// address editing dialog
// ----------------------------------------------------------------------------

class wxVCardAddressDialog : public wxDialog
{
public:
    wxVCardAddressDialog(wxWindow *parent, const wxVCardAddressData& data);

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

    const wxVCardAddressData& GetData() const { return m_data; }

protected:
    wxTextCtrl *m_postoffice,
               *m_extaddr,
               *m_street,
               *m_city,
               *m_region,
               *m_postalcode,
               *m_country;

    wxCheckBox *m_domesticAddr,
               *m_internationalAddr,
               *m_postalAddr,
               *m_parcelAddr,
               *m_homeAddr,
               *m_workAddr;

    wxVCardAddressData m_data;
};

// ----------------------------------------------------------------------------
// phone number editing dialog
// ----------------------------------------------------------------------------

class wxVCardPhoneDialog : public wxDialog
{
public:
    wxVCardPhoneDialog(wxWindow *parent, const wxVCardPhoneData& data);

    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();

    const wxVCardPhoneData& GetData() const { return m_data; }

protected:
    wxTextCtrl *m_number;

    wxCheckBox *m_preferred,
               *m_work,
               *m_home,
               *m_voice,
               *m_fax,
               *m_recorder,
               *m_cellular,
               *m_pager,
               *m_bbs,
               *m_modem,
               *m_car,
               *m_isdn,
               *m_video;

    wxVCardPhoneData m_data;
};

// ----------------------------------------------------------------------------
// event tables
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(wxDateDialog, wxDialog)
    EVT_CALENDAR(-1, wxDateDialog::OnCalendar)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(wxDateBrowseButton, wxButton)
   EVT_BUTTON(-1, wxDateBrowseButton::OnButtonClick)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(wxImageButton, wxBitmapButton)
    EVT_MENU(ImageMenu_Change, wxImageButton::OnChangeImage)
    EVT_MENU(ImageMenu_Clear, wxImageButton::OnClearImage)
    EVT_MENU(ImageMenu_Show, wxImageButton::OnShowImage)

    EVT_BUTTON(-1, wxImageButton::OnButtonClick)

    EVT_RIGHT_UP(wxImageButton::OnRightClick)

    EVT_SIZE(wxImageButton::OnSize)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(wxVCardDialog, wxDialog)
    EVT_BUTTON(Email_Add, wxVCardDialog::OnEmailAdd)
    EVT_BUTTON(Email_Modify, wxVCardDialog::OnEmailModify)
    EVT_BUTTON(Email_Delete, wxVCardDialog::OnEmailDelete)

    EVT_BUTTON(Addr_Add, wxVCardDialog::OnAddrAdd)
    EVT_BUTTON(Addr_Modify, wxVCardDialog::OnAddrModify)
    EVT_BUTTON(Addr_Delete, wxVCardDialog::OnAddrDelete)

    EVT_BUTTON(Phone_Add, wxVCardDialog::OnPhoneAdd)
    EVT_BUTTON(Phone_Modify, wxVCardDialog::OnPhoneModify)
    EVT_BUTTON(Phone_Delete, wxVCardDialog::OnPhoneDelete)

    EVT_UPDATE_UI(Email_Modify, wxVCardDialog::OnUpdateEMail)
    EVT_UPDATE_UI(Email_Delete, wxVCardDialog::OnUpdateEMail)

    EVT_UPDATE_UI_RANGE(Addr_Modify, Addr_Delete,
                        wxVCardDialog::OnUpdateAddress)
    EVT_UPDATE_UI_RANGE(Phone_Modify, Phone_Delete,
                        wxVCardDialog::OnUpdatePhone)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(ImageViewFrame, wxFrame)
    EVT_LEFT_UP(ImageViewFrame::OnClick)
    EVT_PAINT(ImageViewFrame::OnPaint)
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// wxImageButton
// ----------------------------------------------------------------------------

const wxSize wxImageButton::ms_size = wxSize(64, 64);

wxImageButton::wxImageButton(wxWindow *parent, int id)
             : wxBitmapButton(parent, id, GetEmptyBitmap(ms_size),
                              wxDefaultPosition,
                              ms_size)
{
    SetToolTip(_("Click button to change image"));

    m_menuPopup = (wxMenu *)NULL;

    m_dirty = FALSE;
}

wxImageButton::~wxImageButton()
{
    delete m_menuPopup;
}

void wxImageButton::DoSetBitmap(const wxBitmap& bmp)
{
    SetBitmapLabel(bmp);
    SetBitmapFocus(bmp);
    SetBitmapSelected(bmp);

    // update the bitmap on screen
    Refresh();
}

void wxImageButton::ClearImage()
{
    m_image.Destroy();

    DoSetBitmap(GetEmptyBitmap(GetClientSize()));
}

wxBitmap wxImageButton::GetEmptyBitmap(const wxSize& size) const
{
    // construct the bitmap with some help text
    int w = size.x - 10,
        h = size.y - 10;
    wxBitmap bmp(w, h);

    wxMemoryDC dc;
    dc.SelectObject(bmp);
    dc.SetBackground(*wxLIGHT_GREY_BRUSH);
    dc.Clear();

    const wxString text = _("No image");
    int x, y;
    dc.GetTextExtent(text, &x, &y);
    dc.DrawText(text, (w - x) / 2, (h - y) /2);
    dc.SelectObject(wxNullBitmap);

    return bmp;
}

void wxImageButton::SetImage(const wxImage& image)
{
    m_image = image;

    ScaleAndSetImage();
}

void wxImageButton::ScaleAndSetImage()
{
    // construct the bitmap to show a thumbnail of the image: if the image is
    // smaller than our size, leave it as is, otherwise create a thumbnail
    int wBtn, hBtn;
    GetSize(&wBtn, &hBtn);
    wBtn -= 10;
    hBtn -= 10;

    int wImage = m_image.GetWidth(),
        hImage = m_image.GetHeight();

    // never enlarge the image and if we reduce it, do it propertionally in
    // order not to distort it
    wxImage image = m_image;
    float wRatio = (float)wImage / wBtn,
          hRatio = (float)hImage / hBtn;
    float ratio = wRatio > hRatio ? wRatio : hRatio;

    if ( ratio > 1.0 )
    {
        // reduce the image size
        image.Rescale(wImage / ratio, hImage / ratio);
    }

    DoSetBitmap(image.ConvertToBitmap());
}

void wxImageButton::OnSize(wxSizeEvent& event)
{
    if ( m_image.Ok() )
    {
        ScaleAndSetImage();
    }
}

void wxImageButton::OnRightClick(wxMouseEvent& event)
{
    // crete the popup menu if not done yet
    if ( !m_menuPopup )
    {
        m_menuPopup = new wxMenu;
        m_menuPopup->Append(ImageMenu_Change, _("Change image"));
        m_menuPopup->Append(ImageMenu_Clear, _("Clear image"));
        m_menuPopup->AppendSeparator();
        m_menuPopup->Append(ImageMenu_Show, _("Show image full size"));
    }

    // set the menu item state
    m_menuPopup->Enable(ImageMenu_Show, m_image.Ok());

    // and show it
    PopupMenu(m_menuPopup, event.GetX(), event.GetY());
}

void wxImageButton::OnButtonClick(wxCommandEvent& WXUNUSED(event))
{
    SelectImage();
}

void wxImageButton::OnChangeImage(wxCommandEvent& WXUNUSED(event))
{
    SelectImage();
}

void wxImageButton::SelectImage()
{
    wxString filename = wxLoadFileSelector(
                                           _("an image"),
                                           _T("*"),
                                           NULL,
                                           this
                                          );
    if ( filename.length() )
    {
        wxImage image;
        if ( !image.LoadFile(filename) )
        {
            wxLogError(_("Image file couldn't be loaded."));
        }
        else
        {
            SetImage(image);

            m_dirty = TRUE;
        }
    }
}

void wxImageButton::OnClearImage(wxCommandEvent& event)
{
    ClearImage();

    m_dirty = TRUE;
}

void wxImageButton::OnShowImage(wxCommandEvent& event)
{
    // create a new frame holding the image and show it
    ImageViewFrame *frame = new ImageViewFrame(GetParent(),
                                               m_image.ConvertToBitmap());
    frame->Show();
}

// ----------------------------------------------------------------------------
// wxVCardDialog construction
// ----------------------------------------------------------------------------

wxVCardDialog::wxVCardDialog(wxWindow *parent, wxVCard *vcard)
             : wxDialog(parent, -1, _("Edit vCard"),
                        wxDefaultPosition, wxDefaultSize,
                        wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER )
{
    // init members
    // ------------

    m_vcard = vcard;

    m_emailDirty =
    m_addrDirty =
    m_phoneDirty = FALSE;

    // create controls
    // ===============

    // the top sizer contains the notebooks at the top and the buttons in the
    // bottom
    // ----------------------------------------------------------------------

    wxBoxSizer *sizerTop = new wxBoxSizer( wxVERTICAL );

    wxNotebook *notebook = new wxNotebook( this, -1 );
    m_imagelist = new wxImageList(32, 32);
    m_imagelist->Add(wxBitmap(ident_xpm));
    m_imagelist->Add(wxBitmap(network_xpm));
    m_imagelist->Add(wxBitmap(phone_xpm));
    notebook->SetImageList(m_imagelist);

    wxNotebookSizer *sizerNotebook = new wxNotebookSizer( notebook );
    sizerTop->Add(sizerNotebook, 1, wxGROW | wxALL, 5);
    sizerTop->Add(CreateButtonSizer(wxOK | wxCANCEL), 0,
                  wxALIGN_RIGHT | (wxALL & ~wxRIGHT), 10);

    // first page: the identity fields
    // ----------------------------------------------------------------------

    wxPanel *panel = new wxPanel(notebook);
    wxBoxSizer *sizerPanel = new wxBoxSizer(wxVERTICAL);
    wxStaticBoxSizer *sizerName = new wxStaticBoxSizer
                                       (
                                        new wxStaticBox(panel, -1, _("&Name")),
                                        wxVERTICAL
                                       );

    ADD_TEXT_LABEL_TO_SIZER(sizerName, _("&Full name:"), m_fullname);
    sizerName->Add(0, 10);

    // this 4 column sizer has 5 columns, in fact, as there is an extra spacer
    // between the rows 1 and 2
    wxFlexGridSizer *sizer4Col = new wxFlexGridSizer(5, 5, 5);
    sizer4Col->AddGrowableCol(1);
    sizer4Col->AddGrowableCol(4);

    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer4Col, _("&First:"), m_firstname);
    sizer4Col->Add(10, 0);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer4Col, _("&Middle:"), m_middlename);

    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer4Col, _("&Last:"), m_lastname);
    sizer4Col->Add(10, 0);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer4Col, _("&Suffix:"), m_suffixname);

    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer4Col, _("&Prefix:"), m_prefixname);

    sizerName->Add(sizer4Col, 0, wxGROW | wxBOTTOM | wxLEFT | wxRIGHT, 5);

    wxStaticBoxSizer *sizerOrg = new wxStaticBoxSizer
                                       (
                                        new wxStaticBox(panel, -1, _("&Organization")),
                                        wxHORIZONTAL
                                       );

    wxFlexGridSizer *sizer2Col = new wxFlexGridSizer(2, 5, 5);
    sizer2Col->AddGrowableCol(1);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("N&ame:"), m_orgname);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Department:"), m_department);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Title:"), m_title);
    sizerOrg->Add(sizer2Col, 1, wxGROW | (wxALL & ~wxTOP), 5);
    sizerOrg->Add(10, 0);     // not stretchable spacer
    ADD_IMAGE_TO_SIZER(sizerOrg, _("&Logo:"), m_logo);

    // miscellaneous sizer: contains the photo and birthday fields
    wxBoxSizer *sizerMisc = new wxStaticBoxSizer
                                  (
                                   new wxStaticBox(panel, -1, _("&Miscellaneous")),
                                   wxHORIZONTAL
                                  );
    ADD_IMAGE_TO_SIZER(sizerMisc, _("&Photo:"), m_photo);

    sizerMisc->Add(10, 0);    // not stretachable spacer

    m_birthday = new wxTextCtrl(panel, -1);
    sizerMisc->Add(CreateLabelSizer(panel, _("&Birthdate:"), m_birthday),
                   1, wxALIGN_CENTER_VERTICAL | wxALL, 10);
    wxDateBrowseButton *browseBtn = new wxDateBrowseButton
                                        (
                                            m_birthday,
                                            _("Click to choose date")
                                        );
    sizerMisc->Add(browseBtn, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);

    // comments sizer contains only the comments multiline edit ctrl
    wxBoxSizer *sizerComments = new wxStaticBoxSizer
                                    (
                                     new wxStaticBox(panel, -1, _("&Comments")),
                                     wxVERTICAL
                                    );

    m_comments = new wxTextCtrl(panel, -1, wxEmptyString,
                                wxDefaultPosition, wxDefaultSize,
                                wxTE_MULTILINE);
    sizerComments->Add(m_comments, 1, wxALL | wxGROW, 5);
    // don't make the text controls too small
    sizerComments->SetItemMinSize(m_comments, 400, 100);

    // add all sizers to the main panel (vertical) sizer
    sizerPanel->Add(sizerName, 0, wxGROW | wxALL, 5);
    sizerPanel->Add(sizerOrg, 1, wxGROW | wxALL, 5);
    sizerPanel->Add(sizerMisc, 1, wxGROW | wxALL, 5);
    sizerPanel->Add(sizerComments, 1, wxGROW | wxALL, 5);
    sizerPanel->Add(0, 5, 0); // small space below

    panel->SetAutoLayout(TRUE);
    panel->SetSizer(sizerPanel);

    notebook->AddPage(panel, _("Identity"), FALSE, Page_Ident);

    // second page: the network-related fields
    // ---------------------------------------

    panel = new wxPanel(notebook);

    sizerPanel = new wxBoxSizer(wxVERTICAL);
    ADD_TEXT_LABEL_TO_SIZER(sizerPanel, _("&URL:"), m_url);

    wxSizer *sizerEmail = new wxStaticBoxSizer
                              (
                               new wxStaticBox(panel, -1, _("&Email list")),
                               wxHORIZONTAL
                              );

    m_emails = new wxListBox(panel, -1);
    sizerEmail->Add(m_emails, 1, wxGROW | wxALL, 5);
    sizerEmail->Add(CreateLboxButtonsSizer(panel, Email_Add), 0, wxCENTRE);

    sizerPanel->Add(sizerEmail, 1, wxGROW | wxALL, 5);

    wxSizer *sizerKey = new wxStaticBoxSizer
                              (
                               new wxStaticBox(panel, -1, _("&Public key")),
                               wxVERTICAL
                              );
    m_key = new wxTextCtrl(panel, -1, wxEmptyString,
                           wxDefaultPosition, wxDefaultSize,
                           wxTE_MULTILINE);
    sizerKey->Add(m_key, 1, wxALL | wxGROW, 5);

    wxString choices[2];
    choices[0] = _("&PGP");
    choices[1] = _("&X509");
    m_keyKind = new wxRadioBox(panel, -1, _("&Kind"),
                               wxDefaultPosition, wxDefaultSize,
                               2, choices, 2);
    sizerKey->Add(m_keyKind, 0, wxALL, 5);

    sizerPanel->Add(sizerKey, 1, wxGROW | wxALL, 5);

    panel->SetAutoLayout(TRUE);
    panel->SetSizer(sizerPanel);

    notebook->AddPage(panel, _("Network"), FALSE, Page_Network);

    // third page: addresses and phone numbers
    // ---------------------------------------

    panel = new wxPanel(notebook);

    sizerPanel = new wxBoxSizer(wxVERTICAL);

    wxSizer *sizerAddr = new wxStaticBoxSizer
                              (
                               new wxStaticBox(panel, -1, _("&Addresses")),
                               wxHORIZONTAL
                              );

    m_addresses = new wxListBox(panel, -1);
    sizerAddr->Add(m_addresses, 1, wxGROW | wxALL, 5);
    sizerAddr->Add(CreateLboxButtonsSizer(panel, Addr_Add), 0, wxCENTRE);

    sizerPanel->Add(sizerAddr, 1, wxGROW | wxALL, 5);

    wxSizer *sizerPhones = new wxStaticBoxSizer
                               (
                                new wxStaticBox(panel, -1, _("&Phone numbers")),
                                wxHORIZONTAL
                               );

    m_phones = new wxListBox(panel, -1);
    sizerPhones->Add(m_phones, 1, wxGROW | wxALL, 5);
    sizerPhones->Add(CreateLboxButtonsSizer(panel, Phone_Add), 0, wxCENTRE);
    sizerPanel->Add(sizerPhones, 1, wxGROW | wxALL, 5);

    panel->SetAutoLayout(TRUE);
    panel->SetSizer(sizerPanel);

    notebook->AddPage(panel, _("Address"), FALSE, Page_Address);

    // do use the sizer
    // ----------------

    SetAutoLayout( TRUE );
    SetSizer( sizerTop );
    sizerTop->Fit( this );
    sizerTop->SetSizeHints( this );

    // position the dialog
    // -------------------

    CentreOnScreen();
}

// creates a vertical sizer containing the buttons used with listboxes:
// add/modify/remove
wxSizer *wxVCardDialog::CreateLboxButtonsSizer(wxWindow *parent,
                                               int id)
{
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);

    sizer->Add(new wxButton(parent, id++, _("&Add...")), 0, wxCENTRE | wxALL, 5);
    sizer->Add(new wxButton(parent, id++, _("&Modify...")), 0, wxCENTRE | wxALL, 5);
    sizer->Add(new wxButton(parent, id++, _("&Delete")), 0, wxCENTRE | wxALL, 5);

    return sizer;
}

// creates a horizontal box sizer containg a label and text field
wxSizer *wxVCardDialog::CreateLabelSizer(wxWindow *parent,
                                         const wxString& label,
                                         wxControl *control,
                                         bool resizeable)
{
    wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(new wxStaticText(parent, -1, label), 0, wxALIGN_CENTRE | wxRIGHT);
    sizer->Add(control, 1, (resizeable ? wxGROW : 0) | wxALIGN_CENTRE | wxLEFT, 5);

    return sizer;
}

// ----------------------------------------------------------------------------
// wxVCardDialog data transfer
// ----------------------------------------------------------------------------

// transfer data from vCard to the window
bool wxVCardDialog::TransferDataToWindow()
{
    wxString value;

    #define VCARD_PROP_TO_CTRL(prop, ctrl) \
        if ( m_vcard->Get##prop(&value) ) ctrl->SetValue(value);

    // identity page first
    VCARD_PROP_TO_CTRL(FullName, m_fullname);
    {
        wxString familyName, givenName, middleName, prefix, suffix;
        if ( m_vcard->GetName(&familyName, &givenName, &middleName, &prefix, &suffix) )
        {
            m_firstname->SetValue(givenName);
            m_lastname->SetValue(familyName);
            m_middlename->SetValue(middleName);
            m_prefixname->SetValue(prefix);
            m_suffixname->SetValue(suffix);
        }
    }

    {
        wxDateTime dt;
        if ( m_vcard->GetBirthDay(&dt) )
        {
            m_birthday->SetValue(dt.FormatISODate());
        }
    }

    VCARD_PROP_TO_CTRL(BusinessRole, m_title);
    {
        wxString org, dept;
        if ( m_vcard->GetOrganization(&org, &dept) )
        {
            m_orgname->SetValue(org);
            m_department->SetValue(dept);
        }
    }

    {
        wxVCardImage image;
        if ( m_vcard->GetPhoto(&image) )
            m_photo->SetImage(image.GetImage());
        else
            m_photo->ClearImage();
        if ( m_vcard->GetLogo(&image) )
            m_logo->SetImage(image.GetImage());
        else
            m_logo->ClearImage();
    }

    VCARD_PROP_TO_CTRL(Comment, m_comments);

    // network page
    VCARD_PROP_TO_CTRL(URL, m_url);
    {
        void *cookie;
        wxVCardEMail *email = m_vcard->GetFirstEMail(&cookie);
        while ( email )
        {
            m_emails->Append(email->GetEMail());
            delete email;

            email = m_vcard->GetNextEMail(&cookie);
        }
    }

    wxVCard::KeyKind keyKind;
    if ( m_vcard->GetPublicKey(&value, &keyKind) )
    {
        m_key->SetValue(value);
        m_keyKind->SetSelection(keyKind);
    }

    // address page
    {
        void *cookie;
        wxVCardAddress *addr = m_vcard->GetFirstAddress(&cookie);
        while ( addr )
        {
            AddAddress(wxVCardAddressData(addr));

            delete addr;

            addr = m_vcard->GetNextAddress(&cookie);
        }

        wxVCardPhoneNumber *phone = m_vcard->GetFirstPhoneNumber(&cookie);
        while ( phone )
        {
            AddPhone(wxVCardPhoneData(phone));

            delete phone;

            phone = m_vcard->GetNextPhoneNumber(&cookie);
        }
    }

    #undef VCARD_PROP_TO_CTRL

    return TRUE;
}

// transfer data to vCard from the window
bool wxVCardDialog::TransferDataFromWindow()
{
    // do the checks first
    wxDateTime dt;
    wxString birthday = m_birthday->GetValue();
    if ( !!birthday && !dt.ParseDate(birthday) )
    {
        wxLogError(_("Invalid birthday date: '%s'"),
                   m_birthday->GetValue().c_str());

        return FALSE;
    }

    if ( !m_fullname->GetValue() )
    {
        wxLogError(_("Please specify the full name."));

        return FALSE;
    }

    wxString value;

    #define CTRL_TO_VCARD_PROP(prop, ctrl)      \
        if ( ctrl->IsModified() )               \
        {                                       \
            value = ctrl->GetValue();           \
            m_vcard->Set##prop(value);          \
        }

    // identity page
    CTRL_TO_VCARD_PROP(FullName, m_fullname);

    if ( m_lastname->IsModified() ||
         m_firstname->IsModified() ||
         m_middlename->IsModified() ||
         m_prefixname->IsModified() ||
         m_suffixname->IsModified() )
    {
        m_vcard->SetName(m_lastname->GetValue(),
                         m_firstname->GetValue(),
                         m_middlename->GetValue(),
                         m_prefixname->GetValue(),
                         m_suffixname->GetValue());
    }

    if ( !!birthday && m_birthday->IsModified() )
        m_vcard->SetBirthDay(dt);
    CTRL_TO_VCARD_PROP(BusinessRole, m_title);

    if ( m_orgname->IsModified() || m_department->IsModified() )
    {
        m_vcard->SetOrganization(m_orgname->GetValue(),
                                 m_department->GetValue());
    }

    if ( m_photo->IsDirty() )
        m_vcard->SetPhoto(m_photo->GetImage());

    if ( m_logo->IsDirty() )
        m_vcard->SetLogo(m_logo->GetImage());

    CTRL_TO_VCARD_PROP(Comment, m_comments);

    // network page
    CTRL_TO_VCARD_PROP(URL, m_url);

    if ( m_emailDirty )
    {
        m_vcard->ClearEMails();

        size_t count = m_emails->GetCount();
        for ( size_t n = 0; n < count; n++ )
        {
            m_vcard->AddEMail(m_emails->GetString(n));
        }
    }

    m_vcard->SetPublicKey(m_key->GetValue(),
                          (wxVCard::KeyKind)m_keyKind->GetSelection());

    // address page
    if ( m_addrDirty )
    {
        size_t count = m_addrData.GetCount();
        for ( size_t n = 0; n < count; n++ )
        {
            const wxVCardAddressData& d = m_addrData[n];
            m_vcard->AddAddress(d.postoffice,
                                d.extaddr,
                                d.street,
                                d.region,
                                d.postalcode,
                                d.city,
                                d.country,
                                d.flags);
        }
    }

    if ( m_phoneDirty )
    {
        size_t count = m_phoneData.GetCount();
        for ( size_t n = 0; n < count; n++ )
        {
            const wxVCardPhoneData& d = m_phoneData[n];
            m_vcard->AddPhoneNumber(d.number, d.flags);
        }
    }

    #undef CTRL_TO_VCARD_PROP

    return TRUE;
}

// ----------------------------------------------------------------------------
// wxVCardDialog event handler
// ----------------------------------------------------------------------------

// email listbox buttons

void wxVCardDialog::OnEmailAdd(wxCommandEvent& WXUNUSED(event))
{
    // TODO: allow to specify the type of email
    wxString email = wxGetTextFromUser(_("Email address:"),
                                       _("New email address"),
                                       wxEmptyString,
                                       this);
    if ( !email )
    {
        // cancelled by user
        return;
    }

    m_emails->Append(email);

    m_emailDirty = TRUE;
}

void wxVCardDialog::OnEmailModify(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_emails->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    wxString emailOld = m_emails->GetString(sel);

    // TODO: allow to specify the type of email
    wxString email = wxGetTextFromUser(_("Email address:"),
                                       _("Modify the email address"),
                                       emailOld,
                                       this);

    if ( !email || email == emailOld )
        return;

    m_emails->Insert(email, sel);
    m_emails->Delete(sel + 1);  // it was shifted

    m_emailDirty = TRUE;
}

void wxVCardDialog::OnEmailDelete(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_emails->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    m_emails->Delete(sel);

    m_emailDirty = TRUE;
}

// address listbox buttons

void wxVCardDialog::OnAddrAdd(wxCommandEvent& WXUNUSED(event))
{
    wxVCardAddressDialog dlg(this, wxVCardAddressData(NULL));
    if ( dlg.ShowModal() == wxID_OK )
    {
        AddAddress(dlg.GetData());

        m_addrDirty = TRUE;
    }
}

void wxVCardDialog::OnAddrModify(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_addresses->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    wxVCardAddressDialog dlg(this, m_addrData[sel]);
    if ( dlg.ShowModal() == wxID_OK )
    {
        m_addrData[sel] = dlg.GetData();
        m_addresses->SetString(sel, GetAddressLabel(m_addrData[sel]));

        m_addrDirty = TRUE;
    }
}

void wxVCardDialog::OnAddrDelete(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_addresses->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    m_addrData.RemoveAt(sel);
    m_addresses->Delete(sel);

    m_addrDirty = TRUE;
}

// phones listbox buttons

void wxVCardDialog::OnPhoneAdd(wxCommandEvent& WXUNUSED(event))
{
    wxVCardPhoneDialog dlg(this, wxVCardPhoneData(NULL));
    if ( dlg.ShowModal() == wxID_OK )
    {
        AddPhone(dlg.GetData());

        m_phoneDirty = TRUE;
    }
}

void wxVCardDialog::OnPhoneModify(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_phones->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    wxVCardPhoneDialog dlg(this, m_phoneData[sel]);
    if ( dlg.ShowModal() == wxID_OK )
    {
        m_phoneData[sel] = dlg.GetData();
        m_phones->SetString(sel, GetPhoneLabel(m_phoneData[sel]));

        m_phoneDirty = TRUE;
    }
}

void wxVCardDialog::OnPhoneDelete(wxCommandEvent& WXUNUSED(event))
{
    int sel = m_phones->GetSelection();
    wxCHECK_RET( sel != -1, _T("button should be disabled") );

    m_phoneData.RemoveAt(sel);
    m_phones->Delete(sel);

    m_phoneDirty = TRUE;
}

// update UI handlers

void wxVCardDialog::OnUpdateEMail(wxUpdateUIEvent& event)
{
    event.Enable( m_emails->GetSelection() != -1 );
}

void wxVCardDialog::OnUpdateAddress(wxUpdateUIEvent& event)
{
    event.Enable( m_addresses->GetSelection() != -1 );
}

void wxVCardDialog::OnUpdatePhone(wxUpdateUIEvent& event)
{
    event.Enable( m_phones->GetSelection() != -1 );
}

// ----------------------------------------------------------------------------
// address/phone helpers
// ----------------------------------------------------------------------------

wxString wxVCardDialog::GetAddressLabel(const wxVCardAddressData& data) const
{
    // take the address start as the string to appear in the listbox
    wxString label;
    label << data.extaddr << _T(' ')
          << data.street << _T(", ")
          << data.city;
    return label;
}

void wxVCardDialog::AddAddress(const wxVCardAddressData& data)
{
    m_addrData.Add(data);
    m_addresses->Append(GetAddressLabel(data));
}

wxString wxVCardDialog::GetPhoneLabel(const wxVCardPhoneData& data) const
{
    wxString label = data.number;
    label << _T('(')
          << (data.flags & wxVCardPhoneNumber::Home ? _("Home") : _("Work"))
          << _T(')');
    return label;
}

void wxVCardDialog::AddPhone(const wxVCardPhoneData& data)
{
    m_phoneData.Add(data);
    m_phones->Append(GetPhoneLabel(data));
}

// ----------------------------------------------------------------------------
// wxVCardAddressDialog
// ----------------------------------------------------------------------------

wxVCardAddressDialog::wxVCardAddressDialog(wxWindow *parent,
                                           const wxVCardAddressData& data)
                    : wxDialog(NULL, -1, _("Edit Address"),
                               wxDefaultPosition, wxDefaultSize,
                               wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ),
                      m_data(data)
{
    // create controls
    // ---------------

    wxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);

    wxWindow *panel = this; // macros use this variable

    wxStaticBoxSizer *sizerAddr = new wxStaticBoxSizer
                                        (
                                         new wxStaticBox(panel, -1, _("&Address data")),
                                         wxVERTICAL
                                        );

    wxFlexGridSizer *sizer2Col = new wxFlexGridSizer(2, 5, 5);
    sizer2Col->AddGrowableCol(1);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Post office:"), m_postoffice);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Extended:"), m_extaddr);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Street:"), m_street);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&City:"), m_city);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Region:"), m_region);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Postal code:"), m_postalcode);
    ADD_TEXT_LABEL_TO_GRIDSIZER(sizer2Col, _("&Country:"), m_country);

    sizerAddr->Add(sizer2Col, 0, wxGROW | wxALL, 5);
    sizerAddr->SetItemMinSize(sizer2Col, 300, 100);

    sizer2Col = new wxFlexGridSizer(2, 5, 5);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Domestic"), m_domesticAddr);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("International"), m_internationalAddr);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Postal"), m_postalAddr);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Parcel"), m_parcelAddr);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Home"), m_homeAddr);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Work"), m_workAddr);

    wxStaticBoxSizer *sizerAddrType = new wxStaticBoxSizer
                                            (
                                             new wxStaticBox(panel, -1, _("&Type")),
                                             wxVERTICAL
                                            );
    sizerAddrType->Add(sizer2Col, 0, wxGROW);

    sizerTop->Add(sizerAddr, 0, wxGROW | wxALL, 5);
    sizerTop->Add(sizerAddrType, 0, wxGROW | wxALL, 5);
    sizerTop->Add(0, 5, 1); // expandable spacer
    sizerTop->Add(CreateButtonSizer(wxOK | wxCANCEL), 0,
                  wxALIGN_RIGHT | (wxALL & ~wxRIGHT), 10);

    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->Fit(this);
    sizerTop->SetSizeHints(this);

    CentreOnParent();
}

bool wxVCardAddressDialog::TransferDataToWindow()
{
    #define ADDR_TO_DLG(field) m_##field->SetValue(m_data.field)

    ADDR_TO_DLG(postoffice);
    ADDR_TO_DLG(extaddr);
    ADDR_TO_DLG(street);
    ADDR_TO_DLG(region);
    ADDR_TO_DLG(postalcode);
    ADDR_TO_DLG(city);
    ADDR_TO_DLG(country);

    #define SET_ADDR_FLAG(flag, ctrl) \
        if ( m_data.flags & wxVCardAddress::flag ) ctrl->SetValue(TRUE)

    SET_ADDR_FLAG(Domestic, m_domesticAddr);
    SET_ADDR_FLAG(Intl, m_internationalAddr);
    SET_ADDR_FLAG(Postal, m_postalAddr);
    SET_ADDR_FLAG(Parcel, m_parcelAddr);
    SET_ADDR_FLAG(Home, m_homeAddr);
    SET_ADDR_FLAG(Work, m_workAddr);

    #undef SET_ADDR_FLAG
    #undef ADDR_TO_DLG

    return TRUE;
}

bool wxVCardAddressDialog::TransferDataFromWindow()
{
    #define DLG_TO_ADDR(field) m_data.field = m_##field->GetValue()

    DLG_TO_ADDR(postoffice);
    DLG_TO_ADDR(extaddr);
    DLG_TO_ADDR(street);
    DLG_TO_ADDR(region);
    DLG_TO_ADDR(postalcode);
    DLG_TO_ADDR(city);
    DLG_TO_ADDR(country);

    #define GET_ADDR_FLAG(flag, ctrl) \
        if ( ctrl->GetValue() ) m_data.flags |= wxVCardAddress::flag

    m_data.flags = 0;

    GET_ADDR_FLAG(Domestic, m_domesticAddr);
    GET_ADDR_FLAG(Intl, m_internationalAddr);
    GET_ADDR_FLAG(Postal, m_postalAddr);
    GET_ADDR_FLAG(Parcel, m_parcelAddr);
    GET_ADDR_FLAG(Home, m_homeAddr);
    GET_ADDR_FLAG(Work, m_workAddr);

    #undef GET_ADDR_FLAG
    #undef DLG_TO_ADDR

    return TRUE;
}

// ----------------------------------------------------------------------------
// wxVCardPhoneData
// ----------------------------------------------------------------------------

wxVCardPhoneDialog::wxVCardPhoneDialog(wxWindow *parent,
                                           const wxVCardPhoneData& data)
                  : wxDialog(NULL, -1, _("Edit Phone Number"),
                             wxDefaultPosition, wxDefaultSize,
                             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ),
                    m_data(data)
{
    // create controls
    // ---------------

    wxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *sizerPhone = new wxBoxSizer(wxHORIZONTAL);
    wxStaticText *label = new wxStaticText(this, -1, _("&Number:"));
    sizerPhone->Add(label, 0, wxALIGN_CENTRE | wxRIGHT, 5);
    m_number = new wxTextCtrl(this, -1, wxEmptyString);
    sizerPhone->Add(m_number, 1, wxALIGN_CENTRE | wxLEFT, 5);

    wxWindow *panel = this; // macros use this variable

    wxFlexGridSizer *sizer2Col = new wxFlexGridSizer(2, 5, 5);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Work"), m_work);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Home"), m_home);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Voice"), m_voice);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Fax"), m_fax);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Preferred"), m_preferred);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Recorder"), m_recorder);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Cellular"), m_cellular);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("Pa&ger"), m_pager);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&BBS"), m_bbs);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Modem"), m_modem);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Car"), m_car);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&ISDN"), m_isdn);
    ADD_CHECKBOX_TO_SIZER(sizer2Col, _("&Video"), m_video);

    wxStaticBoxSizer *sizerPhoneType = new wxStaticBoxSizer
                                           (
                                            new wxStaticBox(this, -1, _("&Type")),
                                            wxVERTICAL
                                           );
    sizerPhoneType->Add(sizer2Col, 0, wxGROW);

    sizerTop->Add(sizerPhone, 0, wxGROW | wxALL, 5);
    sizerTop->SetItemMinSize(sizerPhone, 250, 20);
    sizerTop->Add(sizerPhoneType, 0, wxGROW | wxALL, 5);
    sizerTop->Add(0, 5, 1); // expandable spacer
    sizerTop->Add(CreateButtonSizer(wxOK | wxCANCEL), 0,
                  wxALIGN_RIGHT | (wxALL & ~wxRIGHT), 10);

    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->Fit(this);
    sizerTop->SetSizeHints(this);

    CentreOnParent();
}

bool wxVCardPhoneDialog::TransferDataToWindow()
{
    m_number->SetValue(m_data.number);

    #define SET_PHONE_FLAG(flag, ctrl) \
        if ( m_data.flags & wxVCardPhoneNumber::flag ) ctrl->SetValue(TRUE)

    SET_PHONE_FLAG(Preferred, m_preferred);
    SET_PHONE_FLAG(Work, m_work);
    SET_PHONE_FLAG(Home, m_home);
    SET_PHONE_FLAG(Voice, m_voice);
    SET_PHONE_FLAG(Fax, m_fax);
    SET_PHONE_FLAG(Messaging, m_recorder);
    SET_PHONE_FLAG(Cellular, m_cellular);
    SET_PHONE_FLAG(Pager, m_pager);
    SET_PHONE_FLAG(BBS, m_bbs);
    SET_PHONE_FLAG(Modem, m_modem);
    SET_PHONE_FLAG(Car, m_car);
    SET_PHONE_FLAG(ISDN, m_isdn);
    SET_PHONE_FLAG(Video, m_video);

    #undef SET_PHONE_FLAG

    return TRUE;
}

bool wxVCardPhoneDialog::TransferDataFromWindow()
{
    m_data.number = m_number->GetValue();

    #define GET_PHONE_FLAG(flag, ctrl) \
        if ( ctrl->GetValue() ) m_data.flags |= wxVCardPhoneNumber::flag

    m_data.flags = 0;

    GET_PHONE_FLAG(Preferred, m_preferred);
    GET_PHONE_FLAG(Work, m_work);
    GET_PHONE_FLAG(Home, m_home);
    GET_PHONE_FLAG(Voice, m_voice);
    GET_PHONE_FLAG(Fax, m_fax);
    GET_PHONE_FLAG(Messaging, m_recorder);
    GET_PHONE_FLAG(Cellular, m_cellular);
    GET_PHONE_FLAG(Pager, m_pager);
    GET_PHONE_FLAG(BBS, m_bbs);
    GET_PHONE_FLAG(Modem, m_modem);
    GET_PHONE_FLAG(Car, m_car);
    GET_PHONE_FLAG(ISDN, m_isdn);
    GET_PHONE_FLAG(Video, m_video);

    #undef GET_PHONE_FLAG

    return TRUE;
}

// ----------------------------------------------------------------------------
// wxDateDialog and wxDateBrowseButton
// ----------------------------------------------------------------------------

wxDateDialog::wxDateDialog(wxWindow *parent,
                           const wxString& title,
                           const wxDateTime& dt)
            : wxDialog(parent, -1, title)
{
    wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
    wxStaticBoxSizer *sizerAddr = new wxStaticBoxSizer
                                      (
                                       new wxStaticBox(this, -1, _("&Choose date")),
                                       wxVERTICAL
                                      );
    m_calendar = new wxCalendarCtrl(this, -1);
    sizerAddr->Add(m_calendar, 1, wxGROW | wxALL, 5);

    sizerTop->Add(sizerAddr, 1, wxGROW | wxALL, 5);
    sizerTop->Add(CreateButtonSizer(wxOK | wxCANCEL), 0,
                  wxALIGN_RIGHT | (wxALL & ~wxRIGHT), 10);

    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->Fit(this);
    sizerTop->SetSizeHints(this);

    Centre();
}

void wxDateDialog::OnCalendar(wxCalendarEvent& event)
{
    EndModal(wxID_OK);
}

void wxDateBrowseButton::DoBrowse()
{
    wxString str = m_text->GetValue();
    wxDateTime dt;
    if ( str.length() )
        dt.ParseDate(str);

    wxDateDialog dialog(this, _("Select the birthday date"), dt);
    if ( dialog.ShowModal() == wxID_OK )
    {
        m_text->SetValue(dialog.GetDate().FormatISODate());
    }
}

// ----------------------------------------------------------------------------
// our public API
// ----------------------------------------------------------------------------

extern bool wxEditVCard(wxVCard *vcard, wxWindow *parent)
{
    wxCHECK_MSG( vcard, FALSE, _T("NULL vCard not allowed in wxEditVCard") );

    wxVCardDialog dlg(parent, vcard);
    return dlg.ShowModal() == wxID_OK;
}

extern wxVCard *wxCreateVCard(wxWindow *parent)
{
    wxVCard *vcard = new wxVCard;
    if ( wxEditVCard(vcard, parent) )
    {
        return vcard;
    }
    else // dialog cancelled
    {
        delete vcard;

        return NULL;
    }
}

